package com.zanxiang.game.back.serve.controller;

import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.back.serve.pojo.dto.OrderReportDTO;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentAppApiOrderSplitLogVO;
import com.zanxiang.game.back.serve.service.*;
import com.zanxiang.module.util.exception.BaseException;
import com.zanxiang.module.util.pojo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

@RestController
@RequestMapping("/tencentAppApi")
@Api("腾讯APP API 回传")
public class TencentAppApiLogController {
    @Autowired
    private IGameTencentAppApiUserService gameTencentAppApiUserService;
    @Autowired
    private IGameTencentAppApiOrderService gameTencentAppApiOrderService;
    @Autowired
    private IGameTencentAppApiOrderSplitLogService gameTencentAppApiOrderSplitLogService;


    @PreAuthorize(permissionKey = "gameBack:tencentAppApi:orderSplits")
    @GetMapping("/orderSplitList/{orderId}")
    public ResultVO<List<GameTencentAppApiOrderSplitLogVO>> orderSplitList(@PathVariable("orderId") String orderId) {
        return ResultVO.ok(gameTencentAppApiOrderSplitLogService.listByOrderNo(Collections.singletonList(orderId)));
    }

    @PreAuthorize(permissionKey = "gameBack:tencentAppApi:orderReport")
    @PostMapping("/orderReport/{ids}/{backMoney}")
    @ApiOperation(value = "腾讯订单手动上报")
    public ResultVO<Boolean> tencentOrderReport(@PathVariable("ids") List<Long> ids, @PathVariable("backMoney") BigDecimal backMoney) {
        OrderReportDTO dto = new OrderReportDTO();
        dto.setOrderIds(ids);
        dto.setSplitOrder(false);
        dto.setBackMoney(backMoney);
        return tencentOrderReport(dto);
    }

    @PreAuthorize(permissionKey = "gameBack:tencentAppApi:orderReport")
    @PostMapping("/orderReport")
    @ApiOperation(value = "腾讯订单手动上报")
    public ResultVO<Boolean> tencentOrderReport(@Validated @RequestBody OrderReportDTO dto) {
        if (CollectionUtils.isEmpty(dto.getOrderIds())) {
            throw new BaseException("参数错误！");
        }
        if (dto.getSplitOrder()) {
            if (CollectionUtils.isEmpty(dto.getSplitMoney())) {
                throw new BaseException("回传金额错误！");
            }
            dto.getSplitMoney().forEach(backMoney -> {
                if (backMoney.compareTo(BigDecimal.ZERO) <= 0) {
                    throw new BaseException("回传金额错误！");
                }
            });
            if (dto.getBetweenMinuteMin() == null || dto.getBetweenMinuteMax() == null
                    || dto.getBetweenMinuteMin() < 1 || dto.getBetweenMinuteMax() < 1
                    || dto.getBetweenMinuteMin().compareTo(dto.getBetweenMinuteMax()) >= 0) {
                throw new BaseException("回传间隔时间错误！");
            }
        } else {
            if (dto.getBackMoney().compareTo(BigDecimal.ZERO) <= 0) {
                throw new BaseException("回传金额错误！");
            }
        }
        return ResultVO.ok(gameTencentAppApiOrderService.tencentOrderReport(dto));
    }

    @PreAuthorize(permissionKey = "gameBack:tencentAppApi:userReport")
    @PostMapping("/userReport/{ids}")
    @ApiOperation(value = "腾讯用户手动上报")
    public ResultVO<Boolean> tencentUserReport(@PathVariable("ids") List<Long> ids) {
        return ResultVO.ok(gameTencentAppApiUserService.tencentUserReport(ids));
    }
}
