package com.zanxiang.game.back.serve.rpc.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.game.back.base.pojo.dto.*;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.back.base.rpc.ITencentAppBackRpc;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.IGameTencentAppCallbackService;
import com.zanxiang.game.back.serve.service.IGameTencentAppOrderService;
import com.zanxiang.game.back.serve.service.IGameTencentAppRoleRegisterService;
import com.zanxiang.game.back.serve.service.IGameTencentAppUserService;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.Objects;

@Slf4j
@DubboService
public class TencentAppBackRpcImpl implements ITencentAppBackRpc {
    @Autowired
    private IGameTencentAppUserService gameTencentAppUserService;
    @Autowired
    private IGameTencentAppRoleRegisterService gameTencentAppRoleRegisterService;
    @Autowired
    private IGameTencentAppOrderService gameTencentAppOrderService;
    @Autowired
    private IGameTencentAppCallbackService gameTencentAppCallbackService;

    @Override
    public ResultVO<Boolean> backUser(TencentUserAppRpcDTO dto) {
        log.error("腾讯 APP用户回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        GameTencentAppUser gameTencentUser = GameTencentAppUser.builder()
                .adAccountId(dto.getAdAccountId())
                .gameId(dto.getGameId())
                .userId(dto.getUserId())
                .channel(dto.getChannel())
                .subscribeTime(dto.getSubscribeTime())
                .registerTime(dto.getRegisterTime())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .isBack(BackStatusEnum.NO.getBackStatus())
                .userActionSetId(dto.getUserActionSetId())
                .createTime(LocalDateTime.now())
                .build();
        gameTencentAppUserService.save(gameTencentUser);
        return ResultVO.ok(true);
        // 激活现在默认不回传了，等创角的时候一起回传
        // return ResultVO.ok(gameTencentUserService.userBack(gameTencentUser, false));
    }

    @Override
    public ResultVO<Boolean> backRoleRegister(TencentRoleRegisterAppRpcDTO dto) {
        log.error("腾讯 APP创角回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        GameTencentAppRoleRegister roleRegister = GameTencentAppRoleRegister.builder()
                .backPolicyId(dto.getBackPolicyId())
                .channel(dto.getChannel())
                .userId(dto.getUserId())
                .gameId(dto.getGameId())
                .adAccountId(dto.getAdAccountId())
                .registerTime(dto.getRegisterTime())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .isBack(BackStatusEnum.NO.getBackStatus())
                .userActionSetId(dto.getUserActionSetId())
                .createTime(LocalDateTime.now())
                .roleId(dto.getRoleId())
                .roleName(dto.getRoleName())
                .build();
        gameTencentAppRoleRegisterService.save(roleRegister);
        // return ResultVO.ok(gameTencentAppRoleRegisterService.roleRegisterBack(roleRegister));
        return ResultVO.ok(true);
    }

    @Override
    public ResultVO<Boolean> backOrder(TencentOrderAppRpcDTO dto) {
        if (Objects.equals(OrderStatusEnum.SUCCESS_PAY.getValue(), dto.getOrderStatus())) {
            log.error("腾讯 APP订单回传收到：{}。", JsonUtil.toString(dto));
        } else {
            log.error("腾讯H5订单回传收到：{}。订单未支付，直接过滤", JsonUtil.toString(dto));
            return ResultVO.ok(true);
        }
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        boolean isFirstOrder = gameTencentAppOrderService.getOne(new LambdaQueryWrapper<GameTencentAppOrder>()
                .select(GameTencentAppOrder::getOrderId)
                .eq(GameTencentAppOrder::getGameId, dto.getGameId())
                .eq(GameTencentAppOrder::getAdAccountId, dto.getAdAccountId())
                .eq(GameTencentAppOrder::getMac, dto.getMac())
                .eq(GameTencentAppOrder::getImei, dto.getImei())
                .eq(GameTencentAppOrder::getOaid, dto.getOaid())
                .eq(GameTencentAppOrder::getAndroidId, dto.getAndroidId())
                .eq(GameTencentAppOrder::getIdfa, dto.getIdfa())
                .eq(GameTencentAppOrder::getCaid, dto.getCaid())
                .eq(GameTencentAppOrder::getOrderStatus, OrderStatusEnum.SUCCESS_PAY.getValue())
                .last("limit 1")
        ) == null;
        //订单保存
        GameTencentAppOrder gameTencentOrder = GameTencentAppOrder.builder()
                .adAccountId(dto.getAdAccountId())
                .userId(dto.getUserId())
                .gameId(dto.getGameId())
                .orderId(dto.getOrderId())
                .channel(dto.getChannel())
                .rechargeMoney(dto.getRechargeMoney())
                .rechargeTime(dto.getRechargeTime())
                .subscribeTime(dto.getSubscribeTime())
                .registerTime(dto.getRegisterTime())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .orderStatus(dto.getOrderStatus())
                .payTime(dto.getPayTime())
                .createTime(LocalDateTime.now())
                .isBack(BackStatusEnum.NO.getBackStatus())
                .backPolicyId(dto.getBackPolicyId())
                .userActionSetId(dto.getUserActionSetId())
                .roleId(dto.getRoleId())
                .roleName(dto.getRoleName())
                .isFirstOrder(isFirstOrder)
                .build();
        gameTencentAppOrderService.save(gameTencentOrder);
        return ResultVO.ok(true);
        // return ResultVO.ok(gameTencentAppOrderService.orderBack(gameTencentOrder));
    }

    @Override
    public ResultVO<String> queryUserAgentFromCallback(TencentAppApiUserAgentQueryRpcDTO dto) {
        /*GameTencentAppCallback callback = gameTencentAppCallbackService.getUserCallback(dto.getGameId(), dto.getImei(), dto.getOaid(), dto.getAndroidId(), dto.getIdfa(), dto.getCaid());
        return ResultVO.ok(callback == null ? null : callback.getAgentKey());*/
        return ResultVO.ok(null);
    }
}
