package com.zanxiang.game.module.base.pojo.dto;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2024-05-23
 * @description : 游戏信息
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class GameDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 超父游戏列表
     */
    private List<GameSupperBean> gameSupperList;

    /**
     * 获取所有游戏列表
     *
     * @return : 返回游戏列表
     */
    public List<GameBean> getAllGameList() {
        if (CollectionUtils.isEmpty(this.gameSupperList)) {
            return Collections.emptyList();
        }
        List<GameBean> gameList = new ArrayList<>();
        this.gameSupperList.forEach(gameSupperBean -> gameList.addAll(gameSupperBean.getGameList()));
        return gameList;
    }

    /**
     * 根据超父获游戏获取超父信息
     *
     * @param gameSupperId : 超父游戏id
     * @return : 返回超父对象
     */
    public GameSupperBean getGameListByGameSupperId(Long gameSupperId) {
        if (CollectionUtils.isEmpty(this.gameSupperList)) {
            return null;
        }
        return this.gameSupperList.stream()
                .filter(gameSupperBean -> Objects.equals(gameSupperBean.getId(), gameSupperId))
                .findFirst().orElse(null);
    }

    /**
     * 根据游戏id获取游戏信息
     *
     * @return : 返回游戏对象
     */
    public GameBean getGameSupperByGameId(Long gameId) {
        if (CollectionUtils.isEmpty(this.gameSupperList)) {
            return null;
        }
        return this.gameSupperList.stream().map(GameSupperBean::getGameList).findFirst()
                .map(gameList -> gameList.stream().filter(gameBean -> Objects.equals(gameBean.getId(), gameId))
                        .findFirst().orElse(null))
                .orElse(null);
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class GameSupperBean implements Serializable {

        private static final long serialVersionUID = 1L;

        /**
         * 游戏id
         */
        private Long id;

        /**
         * CP信息主键id
         */
        private Long cpId;

        /**
         * 游戏名称
         */
        private String name;

        /**
         * 子游戏列表
         */
        private List<GameBean> gameList;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class GameBean implements Serializable {

        private static final long serialVersionUID = 1L;

        /**
         * 游戏id
         */
        private Long id;

        /**
         * 游戏名称
         */
        private String name;

        /**
         * 游戏类型
         */
        private Long category;

        /**
         * 父游戏id
         */
        private Long parentId;

        /**
         * 超父游戏id
         */
        private Long superGameId;
    }
}
