package com.zanxiang.game.module.base.pojo.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户状态
 */
@Getter
@AllArgsConstructor
public enum HttpStatusEnum {

    /**
     * 成功
     */
    SUCCESS(200, "success"),

    /**
     * 失败
     */
    FAIL(400, "failed"),

    /**
     * 服务器异常
     */
    ERROR(500, "服务器异常"),

    /**
     * 未知错误
     */
    UNKNOWN_ERROR(-9999, "未知错误"),

    /**
     * 请先登录
     */
    USER_NO_LOGIN(40400, "登录信息已失效, 请重新登录"),

    /**
     * 用户名为空
     */
    USERNAME_EMPTY(41101, "用户名为空"),

    /**
     * 用户名含敏感字
     */
    USERNAME_SENSITIVE(41102, "用户名含敏感字"),

    /**
     * 用户名已存在
     */
    USERNAME_EXISTS(41103, "用户名已存在"),

    /**
     * 用户名不存在
     */
    USERNAME_NOT_EXISTS(41114, "用户名不存在"),

    /**
     * 用户名或密码错误
     */
    USERNAME_OR_PASSWORD_ERR(41104, "用户名或密码错误"),

    /**
     * 用户名太短
     */
    USERNAME_TOO_SHORT(41105, "用户名太短, 用户名需数字与字母组合,长度为:8-16"),

    /**
     * 用户名太长
     */
    USERNAME_TOO_LONG(41106, "用户名太长, 用户名需数字与字母组合,长度为:8-16"),

    /**
     * 用户名必须为数字字母组合
     */
    USERNAME_BAD_CHAR(41107, "用户名必须为数字字母组合,长度为:8-16"),

    /**
     * 账号停用
     */
    ACCOUNT_HALT(41108, "账号停用, 请联系客服"),

    /**
     * ip封禁
     */
    IP_HALT(41109, "ip封禁, 请联系客服"),

    /**
     * 账户未绑定手机
     */
    ACCOUNT_NOT_BIND_PHONE(41110, "账户未绑定手机"),

    /**
     * 手机号请使用手机注册
     */
    USERNAME_LIKE_MOBILE(41113, "手机号请使用手机注册"),

    /**
     * 密码为空
     */
    PASSWORD_EMPTY(41201, "密码为空"),

    /**
     * 密码必须为数字字母组合
     */
    PASSWORD_BAD_CHAR(41202, "密码必须为数字字母组合,长度为:6-24"),

    /**
     * 密码太长
     */
    PASSWORD_TOO_LONG(41203, "密码太长, 密码需数字与字母组合,长度为:6-24"),

    /**
     * 密码太短
     */
    PASSWORD_TOO_SHORT(41204, "密码太短, 密码需数字与字母组合,长度为:6-24"),

    /**
     * 密码错误
     */
    PASSWORD_ERROR(41205, "密码错误"),

    /**
     * 旧密码错误
     */
    OLD_PASSWORD_ERROR(41206, "旧密码错误"),

    /**
     * 旧密码不可为空
     */
    OLD_PASSWORD_EMPTY(41207, "旧密码不可为空"),

    /**
     * 手机号为空
     */
    PHONE_EMPTY(41301, "手机号为空"),

    /**
     * 手机号格式错误
     */
    PHONE_ERROR(41302, "手机号格式错误"),

    /**
     * 未绑定手机号
     */
    PHONE_NOT_BIND(41304, "未绑定手机号"),

    /**
     * 账号已绑定该手机号
     */
    PHONE_IS_BIND(41305, "账号已绑定该手机号"),

    /**
     * 验证码发送过于频繁
     */
    PHONE_SEND_MORE(41306, "验证码发送过于频繁"),

    /**
     * 验证码发送失败
     */
    PHONE_SEND_ERROR(41307, "验证码发送失败"),

    /**
     * 验证码已发送请稍后再试
     */
    PHONE_HAS_SEND(41308, "验证码已发送请稍后再试"),

    /**
     * 手机号已注册
     */
    PHONE_IS_REG(41309, "手机号已注册"),

    /**
     * 手机号未注册
     */
    PHONE_NOT_REG(41310, "手机号未注册"),

    /**
     * 请先校验原有手机号
     */
    PHONE_NOT_CHECK(41311, "请先校验原有手机号"),

    /**
     * 手机号绑定账号过多
     */
    PHONE_BIND_TOO_MUCH(41312, "手机号绑定账号过多"),

    /**
     * 手机号错误
     */
    PHONE_NUM_ERROR(41313, "手机号错误"),

    /**
     * 验证码错误
     */
    CODE_ERROR(41601, "验证码错误"),

    /**
     * 验证码类型错误
     */
    CODE_TYPE_ERROR(41602, "验证码类型错误"),

    /**
     * 验证码类型错误
     */
    CODE_NOT_SEND(41603, "验证码类型错误"),

    /**
     * 验证码已过期
     */
    CODE_IS_EXPIRED(41604, "验证码已过期"),

    /**
     * 配置错误
     */
    CONFIG_ERROR(41699, "配置错误"),

    /**
     * 参数非法
     */
    INVALID_PARAMS(1000, "参数非法"),

    /**
     * 订单游戏id为空
     */
    ORDER_GAME_ID_IS_NULL(41501, "订单游戏id为空"),

    /**
     * 订单不存在
     */
    ORDER_NO_FIND(41502, "订单不存在"),

    /**
     * 玩家充值未达标
     */
    ORDER_AMOUNT_IS_NULL(41503, "订单金额为空"),

    /**
     * 生成access_token失败
     */
    ACCESS_TOKEN_CREATE_ERROR(41504, "生成access_token失败"),

    /**
     * 米大师连接失败
     */
    MDS_CONNECT_ERROR(41505, "米大师连接失败"),

    /**
     * 获取配置异常
     */
    GET_CONFIG_ERROR(41507, "获取配置异常"),

    /**
     * 游戏用户未创建成功
     */
    GAME_USER_UN_CREATE(41511, "更新游戏角色时，检测为先创建游戏用户"),

    /**
     * 实名认证名字不合法
     */
    CARD_NAME_ERROR(41512, "名字输入错误"),

    /**
     * 实名认证身份证号不合法
     */
    CARD_ID_ERROR(41513, "身份证号输入错误"),

    /**
     * 已实名认证, 不可重复操作
     */
    REPEAT_OPERATE(41514, "已实名认证, 不可重复操作"),

    /**
     * 实名认证失败
     */
    AUTHENTICATION_FAIL(41515, "实名认证失败"),

    /**
     * 参数为空
     */
    PARAM_IS_NULL(41500, "参数为空：");

    /**
     * 回传类型
     */
    private final Integer code;

    /**
     * 回传名称
     */
    private final String msg;
}
