package com.zanxiang.game.module.manage.pojo.params;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;

/**
 * @author : lingfeng
 * @time : 2022-07-12
 * @description : 游戏虚拟支付提交参数
 */
@Data
public class GamePayWayAddUpdateParam {

    /**
     * 主键id
     */
    @ApiModelProperty(notes = "主键id, 获取接口拿到的id是啥就传啥")
    private Long id;

    /**
     * 游戏id
     */
    @NotNull(message = "游戏id不可为空")
    @ApiModelProperty(notes = "游戏id, 任何时候都能拿到, 不可为空")
    private Long gameId;

    /**
     * 支付方式id
     */
    @NotNull(message = "支付方式id不可为空")
    @ApiModelProperty(notes = "支付方式id")
    private Long payWayId;

    /**
     * 支付类型id
     */
    @ApiModelProperty(notes = "支付类型id")
    private Long payDeviceId;

    /**
     * 描述
     */
    @ApiModelProperty(notes = "备注")
    private String remark;

    /**
     * 米大师支付配置
     */
    @ApiModelProperty(notes = "米大师支付配置")
    private MiPayConfigBean miPayConfig;

    /**
     * 微信支付配置
     */
    @ApiModelProperty(notes = "微信支付配置")
    private WxPayConfigBean wxPayConfig;

    /**
     * 支付宝支付配置
     */
    @ApiModelProperty(notes = "支付宝支付配置")
    private AliPayConfigBean aliPayConfig;

    @Data
    public static class MiPayConfigBean {

        /**
         * 米大师支付应用id
         */
        @ApiModelProperty(notes = "米大师支付应用id")
        private String appId;

        /**
         * 米大师现网AppKey
         */
        @ApiModelProperty(notes = "米大师现网AppKey")
        private String appKey;

        /**
         * 米大师沙箱AppKey
         */
        @ApiModelProperty(notes = "米大师沙箱AppKey")
        private String appKeyDev;
    }

    @Data
    public static class WxPayConfigBean {

        /**
         * 支付盒子id
         */
        @ApiModelProperty(notes = "支付盒子id")
        private Long payBoxId;

        /**
         * 客服消息卡片图片地址
         */
        @ApiModelProperty(notes = "客服支付, 消息卡片图片地址")
        private String thumbUrl;

        /**
         * 微信应用appId
         */
        @ApiModelProperty(notes = "支付应用appId")
        private String appId;

        /**
         * 商户号
         */
        @ApiModelProperty(notes = "商户号")
        private String merchantNo;

    }

    @Data
    public static class AliPayConfigBean {

        /**
         * 支付应用appId
         */
        @ApiModelProperty(notes = "支付应用appId")
        private String appId;
    }
}
