package com.zanxiang.game.module.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.game.module.manage.pojo.dto.GameUserRoleDTO;
import com.zanxiang.game.module.manage.pojo.params.GameUserRoleListParam;
import com.zanxiang.game.module.manage.pojo.vo.GameUserRoleListVO;
import com.zanxiang.game.module.manage.pojo.vo.GameUserRoleVO;
import com.zanxiang.game.module.mybatis.entity.GameUserRole;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author : lingfeng
 * @time : 2022-07-01
 * @description : 玩家角色
 */
public interface IGameUserRoleService extends IService<GameUserRole> {

    /**
     * 获取用户最近的游戏角色
     *
     * @param userId 用户id
     * @param gameId 游戏id
     * @return {@link GameUserRole}
     */
    GameUserRole getLastGameUserRoleName(Long userId, Long gameId);

    /**
     * 根据角色id查询
     *
     * @param roleId : 角色id
     * @param gameId 游戏id
     * @param userId 用户id
     * @return {@link GameUserRoleDTO}
     */
    GameUserRoleDTO getByRoleId(Long userId, Long gameId, String roleId);

    /**
     * 单个用户的角色列表
     *
     * @param userId : 用户id
     * @return {@link List}<{@link GameUserRoleVO}>
     */
    List<GameUserRoleVO> roleList(Long userId);

    /**
     * 查询玩家角色列表
     *
     * @param param : 角色列表查询参数
     * @return {@link IPage}<{@link GameUserRoleListVO}>
     */
    IPage<GameUserRoleListVO> list(GameUserRoleListParam param);

    /**
     * 游戏用户角色excel
     *
     * @param param    参数
     * @param response 响应
     */
    void getGameUserRoleExcel(GameUserRoleListParam param, HttpServletResponse response);
}
